require 'cocoapods/installer'
require 'cocoapods/resolver'
require 'cocoapods-cafswitcher/Main'
module Pod

  class Podfile
    # add flag
    # :isSource => true
    def self.isSource
      :isSource
    end

    # add flag
    # :isFramework
    def self.isFramework
      :isFramework
    end

    # add flag
    # :isDebugPackage
    def self.isDebugPackage
      :isDebugPackage
    end

    # add flag
    # :isReleasePackage
    def self.isReleasePackage
      :isReleasePackage
    end

    @@sourceList = []

    def self.sourceList
      @@sourceList
    end

    @@unspecifiedList = []

    def self.unspecifiedList
      @@unspecifiedList
    end

    @@frameworkList = []

    def self.frameworkList
      @@frameworkList
    end

    @@debugPackageList = Array.new

    def Podfile.debugPackageList
      @@debugPackageList
    end

    @@releasePackageList = Array.new

    def Podfile.releasePackageList
      @@releasePackageList
    end

    # all the name of Pod
    @@allPodName = []

    def self.allPodName
      @@allPodName
    end

    @@gitPathList = []

    def self.gitPathList
    @@gitPathList
    end

    module DSL
      # Enable code for all pods
      # it has a lower priority to other binary settings
      def all_source!
        ENV['IS_SOURCE'] = '1'
      end

      # Enable release package for all pods
      # it has a lower priority to other binary settings
      def all_release_package!
        ENV['IS_RELEASE'] = '1'
      end

    end

    class TargetDefinition

      def parse_source_framework_key(name, requirements)
        options = requirements.last
        if not options.is_a?(Hash)
          Pod::Podfile.unspecifiedList << name
          return requirements
        end
        isFramework = options.key?(Pod::Podfile.isFramework)
        isSource = options.key?(Pod::Podfile.isSource)
        if isSource or (isSource and isFramework)
          Pod::Podfile.sourceList << name
        elsif isFramework
          Pod::Podfile.frameworkList << name
        else
          Pod::Podfile.unspecifiedList << name
        end
        requirements.pop if options.empty?
      end

      def parse_debug_release_package(name, requirements)
        options = requirements.last
        if not options.is_a?(Hash)
          Pod::Podfile.debugPackageList << name
          return requirements
        end
        isDebugPackage = options.key?(Pod::Podfile.isDebugPackage)
        isReleasePackage = options.key?(Pod::Podfile.isReleasePackage)
        if (isDebugPackage and isReleasePackage) or isReleasePackage
          Pod::Podfile.releasePackageList << name
        else
          Pod::Podfile.debugPackageList << name
        end
        requirements.pop if options.empty?
      end

      def parse_git_path(name, requirements)
        options = requirements.last
        return requirements unless options.is_a?(Hash)

        isGitPath = options.key?(:git)
        Pod::Podfile.gitPathList << name if isGitPath

        isPath = options.key?(:path)
        Pod::Podfile.gitPathList << name if isPath
        requirements.pop if options.empty?
      end

      # ---- patch method ----
      # We want modify `store_pod` method, but it's hard to insert a line in the
      # implementation. So we patch a method called in `store_pod`.
      old_method = instance_method(:parse_inhibit_warnings)

      define_method(:parse_inhibit_warnings) do |name, requirements|
        parse_source_framework_key(name, requirements)
        parse_debug_release_package(name, requirements)
        parse_git_path(name, requirements)
        Pod::Podfile.allPodName << name
        old_method.bind(self).call(name, requirements)
      end

      def deep_copy(o)
        Marshal.load(Marshal.dump(o))
      end

      def clean(name, requirements)
        tmp_hash = deep_copy(requirements)
        options = tmp_hash.last
        return tmp_hash unless options.is_a?(Hash)

        isFramework = options.delete(Pod::Podfile.isFramework)
        isSource = options.delete(Pod::Podfile.isSource)
        isDebugPackage = options.delete(Pod::Podfile.isDebugPackage)
        isReleasePackage = options.delete(Pod::Podfile.isReleasePackage)
        tmp_hash.pop if options.empty?
        tmp_hash
      end


      define_method(:store_pod) do |name, *requirements|
        return if parse_subspecs(name, requirements) # This parse method must be called first
        parse_inhibit_warnings(name, requirements)
        parse_configuration_whitelist(name, requirements)

        # 去掉添加的字段
        tmp_hash = clean(name, requirements)
        pod = if tmp_hash && !tmp_hash.empty?
          { name => tmp_hash }
        else
          name
              end

        get_hash_value('dependencies', []) << pod
        nil
      end

    end
  end


  class Specification

      def self.addDependency(spec, subspec)
        dependence_note = []
        if subspec.dependencies
          subspec.dependencies.each do |dependency|
            next unless dependency.name =~ /^(?i)#{spec.name}\//
            dependency_subspec = nil?
            spec.subspecs.each do |temSubspec|
              if temSubspec.name == dependency.name
                dependency_subspec = temSubspec
              end
            end
            if dependency_subspec
              dependence_note = dependence_note + addDependency(spec, dependency_subspec)
            end
            dependence_note << dependency.name
          end
        end
        dependence_note.uniq
      end

      # 处理subspec的内部依赖为相应的形式。
      old_specification_from_file = method(:from_file)
      define_singleton_method(:from_file) do |path, subspec_name = nil|
        spec = old_specification_from_file.call(path, subspec_name)
        need_reload = false
        if spec.subspecs
          source_dependence_note =[]
          framework_dependence_note = []
          # 如果指定的是库本身，则全部subspec改变。
          if Pod::Podfile.frameworkList.include?(spec.name)
            spec.subspecs.each do |subspec|
              framework_dependence_note << subspec.name
            end
          end

          if Pod::Podfile.sourceList.include?(spec.name)
            spec.subspecs.each do |subspec|
              source_dependence_note << subspec.name
            end
          end

          if !Pod::Podfile.frameworkList.include?(spec.name) and !Pod::Podfile.sourceList.include?(spec.name)
            spec.subspecs.each do |subspec|
              next unless subspec.dependencies
              # 如果指定的是susbpec库，则subspec 的dependency改变。

              if Pod::Podfile.frameworkList.include?(subspec.name)
                framework_dependence_note = framework_dependence_note + addDependency(spec, subspec)
              end
              if Pod::Podfile.sourceList.include?(subspec.name)
                source_dependence_note = source_dependence_note + addDependency(spec, subspec)
              end
            end
          end

          framework_dependence_note.uniq.each do |note|
            need_reload = true
            cleanENV note
            setEnv note, '_FRAMEWORK'
          end

          source_dependence_note.uniq.each do |note|
            need_reload = true
            cleanENV note
            setEnv note, '_SOURCE'
          end


        end
        if need_reload
          spec = old_specification_from_file.call(path, subspec_name)
        end
        spec
      end
    end




  # class Lockfile
  #
  #   old_detect_changes_with_podfile = instance_method(:detect_changes_with_podfile)
  #
  #   define_method(:detect_changes_with_podfile) do |podfile|
  #     pods_by_state = old_detect_changes_with_podfile.bind(self).(podfile)
  #     Pod::UI.puts Pod::Podfile.changedSubPods
  #     Pod::Podfile.changedSubPods.each do |changedSubPod|
  #       changedPod = changedSubPod.split("/")[0]
  #       if changedSubPod =~ /^(?i)LJ/ or changedSubPod =~ /^(?i)Lianjia/
  #         if pods_by_state.has_key?(:changed) and !pods_by_state[:changed].include? changedPod
  #           pods_by_state[:changed] << changedPod
  #           if pods_by_state.has_key?(:unchanged) and pods_by_state[:unchanged].include? changedPod
  #               pods_by_state[:unchanged].delete changedPod
  #           end
  #         end
  #       end
  #     end
  #     pods_by_state
  #   end
  #
  # end
  #

  # class Dependency
  #   old_method = instance_method(:compatible?)
  #
  #   define_method(:compatible?) do |other|
  #     # 改变状态
  #     old_method.bind(self).call(other)
  #   end
  # end
  
  # class Resolver
  #
  #
  #   old_method = instance_method(:resolve)
  #
  #   define_method(:resolve) do ||
  #     state = old_method.bind(self).call()
  #     # 找到所有的依赖，如果不是
  #     changedSubPods = []
  #     all_pod_name = []
  #     Pod::Podfile.allPodName.each do |subPod|
  #       podName = subPod.split('/')[0]
  #       all_pod_name << podName
  #     end
  #     @activated.vertices.each do |key, value|
  #       podName = key.split('/')[0]
  #       next if all_pod_name.include?(podName)
  #       changedSubPods << key
  #     end
  #     # 清空缓存
  #     clean changedSubPods
  #   end
  # end
end
